# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

import os
import requests
import pytest
from util import get_raw_http_response
from pytest_httpbin import serve
from httpbin import app as httpbin_app


def test_content_type_header_not_automatically_added(httpbin):
    """
    The server was automatically adding this for some reason, see issue #5
    """
    resp = requests.get(httpbin + '/headers').json()['headers']
    assert 'Content-Type' not in resp


def test_unicode_data(httpbin):
    """
    UTF-8 was not getting recognized for what it was and being encoded as if it
    was binary, see issue #7
    """
    resp = requests.post(
        httpbin + '/post',
        data=u'оживлённым'.encode('utf-8'),
        headers={
            'content-type': 'text/html; charset=utf-8',
        }
    )
    assert resp.json()['data'] == u'оживлённым'


def test_server_should_be_http_1_1(httpbin):
    """
    The server should speak HTTP/1.1 since we live in the future, see issue #6
    """
    resp = get_raw_http_response(httpbin.host, httpbin.port, '/get')
    assert resp.startswith(b'HTTP/1.1')

def test_dont_crash_on_certificate_problems(httpbin_secure):
    with pytest.raises(Exception):
        # this request used to hang
        requests.get(
            httpbin_secure + '/get',
            verify = True,
            cert=__file__
        )
    # and this request would never happen
    requests.get(
        httpbin_secure + '/get',
        verify = True,
    )


@pytest.mark.parametrize('protocol', ('http', 'https'))
def test_fixed_port_environment_variables(protocol):
    """
    Note that we cannot test the fixture here because it is session scoped
    and was already started. Thus, let's just test a new Server instance.
    """
    if protocol == 'http':
        server_cls = serve.Server
        envvar = 'HTTPBIN_HTTP_PORT'
    elif protocol == 'https':
        server_cls = serve.SecureServer
        envvar = 'HTTPBIN_HTTPS_PORT'
    else:
        raise RuntimeError('Unexpected protocol param: {0}'.format(protocol))

    # just have different port to avoid adrress already in use
    # if the second test run too fast after the first one (happens on pypy)
    port = 12345 + len(protocol)

    try:
        envvar_original = os.environ.get(envvar, None)
        os.environ[envvar] = str(port)
        server = server_cls(application=httpbin_app)
        assert server.port == port
    finally:
        # if we don't do this, it blocks:
        try:
            server.start()
            server.stop()
        except UnboundLocalError:
            pass

        # restore the original environ:
        if envvar_original is None:
            del os.environ[envvar]
        else:
            os.environ[envvar] = envvar_original



import unittest
import requests
import pytest_httpbin


def test_httpbin_gets_injected(httpbin):
    assert httpbin.url


def test_httpbin_accepts_get_requests(httpbin):
    assert requests.get(httpbin.url + '/get').status_code == 200


def test_httpbin_secure_accepts_get_requests(httpbin_secure):
    assert requests.get(httpbin_secure.url + '/get').status_code == 200


def test_httpbin_secure_accepts_lots_of_get_requests(httpbin_secure):
    for i in range(10):
        assert requests.get(httpbin_secure.url + '/get').status_code == 200


def test_httpbin_accepts_lots_of_get_requests_in_single_session(httpbin):
    session = requests.Session()

    for i in range(10):
        assert session.get(httpbin.url + '/get').status_code == 200


def test_httpbin_both(httpbin_both):
    # this test will get called twice, once with an http url, once with an
    # https url
    assert requests.get(httpbin_both.url + '/get').status_code == 200


def test_httpbin_join(httpbin):
    assert httpbin.join('foo') == httpbin.url + '/foo'


def test_httpbin_str(httpbin):
    assert httpbin + '/foo' == httpbin.url + '/foo'

def test_chunked_encoding(httpbin_both):
    assert requests.get(httpbin_both.url + '/stream/20').status_code == 200

@pytest_httpbin.use_class_based_httpbin
@pytest_httpbin.use_class_based_httpbin_secure
class TestClassBassedTests(unittest.TestCase):
    def test_http(self):
        assert requests.get(self.httpbin.url + '/get').status_code == 200

    def test_http_secure(self):
        assert requests.get(self.httpbin_secure.url + '/get').status_code == 200



import socket


def get_raw_http_response(host, port, path):

    CRLF = b"\r\n"

    request = [
        b"GET " + path.encode('ascii') + b" HTTP/1.1",
        b"Host: " + host.encode('ascii'),
        b"Connection: Close",
        b"",
        b"",
    ]

    # Connect to the server
    s = socket.socket()
    s.connect((host, port))

    # Send an HTTP request
    s.send(CRLF.join(request))

    # Get the response (in several parts, if necessary)
    response = b''
    buffer = s.recv(4096)
    while buffer:
        response += buffer
        buffer = s.recv(4096)

    return response



from setuptools import setup, find_packages
import codecs
import os
import re

with open("pytest_httpbin/version.py") as f:
    code = compile(f.read(), "pytest_httpbin/version.py", 'exec')
    exec(code)

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the relevant file
with codecs.open(os.path.join(here, 'DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="pytest-httpbin",

    # There are various approaches to referencing the version. For a discussion,
    # see http://packaging.python.org/en/latest/tutorial.html#version
    version=__version__,

    description="Easily test your HTTP library against a local copy of httpbin",
    long_description=long_description,

    # The project URL.
    url='https://github.com/kevin1024/pytest-httpbin',

    # Author details
    author='Kevin McCarthy',
    author_email='me@kevinmccarthy.org',

    # Choose your license
    license='MIT',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    # What does your project relate to?
    keywords='pytest-httpbin testing pytest httpbin',
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    include_package_data = True, # include files listed in MANIFEST.in
    install_requires = ['httpbin','six'],

    # the following makes a plugin available to pytest
    entry_points = {
        'pytest11': [
            'httpbin = pytest_httpbin.plugin',
        ]
    },
)
